/*
	Copyright (c) 2005 by Sean Kent (ryoohki@gmail.com). All rights reserved.
	Permission to use, copy, modify and distribute this software for any
	purpose is hereby granted without fee, provided this copyright and
	permissions notice appear in all copies and derivatives.
*/

using System;
using System.Runtime.InteropServices;

namespace Kent.Library {
	public sealed class DynamicLibrary : IDisposable {
		public DynamicLibrary(string libraryFileName) {
			this.libraryFileName = libraryFileName;
			if (libraryFileName == null)
				throw new ArgumentNullException("Library name must not be null.");

			libraryHandle = LoadLibrary(libraryFileName);

			if (libraryHandle == IntPtr.Zero) {
				int lastError = Marshal.GetLastWin32Error();
				throw new InvalidOperationException("LoadLibrary failed with error code: " + lastError);
			}
		}

		public void GetDelegateByName<T>(string functionName, out T del) {
			if (disposed)
				throw new InvalidOperationException("Object has already been disposed.");

			IntPtr functionPointer = GetProcAddress(libraryHandle, functionName);
			object delegateObject = Marshal.GetDelegateForFunctionPointer(functionPointer, typeof(T));
			del = (T)delegateObject;
		}

		public void Dispose() {
			if (!disposed) {
				FreeLibrary(libraryHandle);
				disposed = true;
			}
		}

		~DynamicLibrary() {
			Dispose();
		}

		public string LibraryFileName { get { return libraryFileName; } }

		[DllImport("Kernel32", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall, SetLastError=true)]
		private static extern IntPtr LoadLibrary(string libraryFileName);

		[DllImport("Kernel32", CallingConvention = CallingConvention.StdCall, SetLastError = true)]
		private static extern void FreeLibrary(IntPtr libraryHandle);

		[DllImport("Kernel32", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall, SetLastError = true)]
		private static extern IntPtr GetProcAddress(IntPtr handle, string libraryFileName);

		private IntPtr libraryHandle;
		private string libraryFileName;
		private bool disposed = false;
	}
}
