/* Copyright John-Philip "Seriema" Johansson (seriema@users.sourceforge.net)
 * released under the MIT license */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ThumbviewSample {
    /// <summary>
    /// This is a sample class showing how to use ExtractorConnector to hook up
    /// to a ThumbView Extractor and use it to create thumbnails for your
    /// application. Search for uses of "ExtractorConnector" to find out how it's used.
    /// </summary>
	public partial class ViewerForm : Form {

        public const string extractorDLLPath=@"extractors";

		public ViewerForm() {
			InitializeComponent();
		}

		private void button1_Click(object sender, EventArgs e) {
			openFileDialog1.FileName = string.Empty;
			if (openFileDialog1.ShowDialog() != DialogResult.OK)
				return;

			Size originalSize;
			ThumbviewConnector.PixelFormat format;
			connector.GetInfo(openFileDialog1.FileName, out format, out originalSize);
			imageName.Text = openFileDialog1.FileName;
			bitsPerPixel.Text = format.ToString();
			dimensions.Text = originalSize.ToString();

            Size imageBoxSize = sampleImage.Size; // So we can reference it below.
            sampleImage.Image = connector.ExtractThumbnail(openFileDialog1.FileName, ThumbviewConnector.PixelFormat.RGBA32, ref imageBoxSize);
		}

        private void Abort(string message)
        {
            MessageBox.Show(this, message, "Error");
            Application.Exit();
        }

		private void Form1_Load(object sender, EventArgs e) {
            try
            {
                string[] foundExtractors = System.IO.Directory.GetFiles(extractorDLLPath, "*.dll");
                if (foundExtractors.Length == 0)
                {
                    Abort("No ThumbView Extractors found in subdirectory '" + extractorDLLPath + "'. Please download them from the ThumbView website (http://thumbview.sf.net), note: they are available in the full ThumbView download page (not the Lite version).");
                    return;
                }
                foreach (string extractor in foundExtractors)
                    extractorDLLs.Items.Add(extractor);
            }
            catch (System.IO.DirectoryNotFoundException)
            {
                Abort("Subdirectory '"+extractorDLLPath+"' not found. Please create it and place wanted ThumbView Extractors there.");
                return;
            }
            catch (Exception exception)
            {
                Abort(exception.ToString());
                return;
            }

            extractorDLLs.SelectedIndex = 0;
            extractorDLLs_SelectedIndexChanged(null, null);
		}

		private ThumbviewConnector.ExtractorConnector connector;

        private void extractorDLLs_SelectedIndexChanged(object sender, EventArgs e)
        {
            connector = new ThumbviewConnector.ExtractorConnector((string)extractorDLLs.SelectedItem);
            extractorName.Text = connector.Name;
            extractorVersion.Text = connector.Version.ToString();

            openFileDialog1.Filter = "Image types|";
            foreach (string extension in connector.SupportedFileTypes)
            {
                openFileDialog1.Filter += "*." + extension + ";";
                extractorFileTypes.Items.Add("." + extension);
            }            

        }
	}
}